==> stypes-0.1.1a3.tar.gz: stypes-0.1.1a3/stypes/util.py <==
from functools import partial

class InvalidSpecError(Exception):
    pass

class UnconvertedValue(object):
    def __init__(self, string, reason):
        self.string = string
        self.reason = reason

    def __nonzero__(self):
        return False

    def __str__(self):
        return "%s, given=%r" % (self.reason, self.string)

    def __repr__(self):
        return '<UnconvertedValue string=%r reason=%r>' % (self.string,
                self.reason)

class imemoize(object):
    """cache the return value of a method
    
    This class is meant to be used as a decorator of methods. The return value
    from a given method invocation will be cached on the instance whose method
    was invoked. All arguments passed to a method decorated with memoize must
    be hashable.
    
    If a memoized method is invoked directly on its class the result will not
    be cached. Instead the method will be invoked like a static method:
    class Obj(object):
        @memoize
        def add_to(self, arg):
            return self + arg
    Obj.add_to(1) # not enough arguments
    Obj.add_to(1, 2) # returns 3, result is not cached

    http://code.activestate.com/recipes/577452-a-memoize-decorator-for-instance-methods/
    """
    def __init__(self, func):
        self.func = func

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self.func
        return partial(self, obj)

    def __call__(self, *args, **kw):
        obj = args[0]
        try:
            cache = obj.__cache
        except AttributeError:
            cache = obj.__cache = {}
        key = (self.func, args[1:], frozenset(kw.items()))
        try:
            res = cache[key]
        except KeyError:
            res = cache[key] = self.func(*args, **kw)
        return res

==> stypes-0.23.2.tar.gz: stypes-0.23.2/stypes/util.py <==
from builtins import object
from functools import partial

class InvalidSpecError(Exception):
    pass

class UnconvertedValue(object):
    def __init__(self, string, reason):
        self.string = string
        self.reason = reason

    def __bool__(self):
        return False

    def __str__(self):
        return "%s, given=%r" % (self.reason, self.string)

    def __repr__(self):
        return '<UnconvertedValue string=%r reason=%r>' % (self.string,
                self.reason)

class imemoize(object):
    """cache the return value of a method
    
    This class is meant to be used as a decorator of methods. The return value
    from a given method invocation will be cached on the instance whose method
    was invoked. All arguments passed to a method decorated with memoize must
    be hashable.
    
    If a memoized method is invoked directly on its class the result will not
    be cached. Instead the method will be invoked like a static method:
    class Obj(object):
        @memoize
        def add_to(self, arg):
            return self + arg
    Obj.add_to(1) # not enough arguments
    Obj.add_to(1, 2) # returns 3, result is not cached

    http://code.activestate.com/recipes/577452-a-memoize-decorator-for-instance-methods/
    """
    def __init__(self, func):
        self.func = func

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self.func
        return partial(self, obj)

    def __call__(self, *args, **kw):
        obj = args[0]
        try:
            cache = obj.__cache
        except AttributeError:
            cache = obj.__cache = {}
        key = (self.func, args[1:], frozenset(list(kw.items())))
        try:
            res = cache[key]
        except KeyError:
            res = cache[key] = self.func(*args, **kw)
        return res
